@@ -5,6 +5,13 @@ import scipy.constants as spc
 import mpmath
 from itertools import chain
 
+supported_natural_constants = ('pi', '_hbar', '_NA', '_c', '_h', '_R', '_k', '_e', '_me', '_mp')
+natural_constant_values = tuple([str(i) for i in (spc.pi, spc.hbar, spc.N_A, spc.c, spc.h, spc.R, spc.k, spc.e,
+                                          spc.electron_mass, spc.proton_mass)])
+supported_functions = ('sinh(', 'cosh(', 'tanh(', 'exp(', 'sin(', 'cos(', 'tan(', 'ln(', 'rad(', 'deg(', 'log(',
+                      'sqrt(', 'arcsin(', 'arccos(', 'arctan(', 'arcsinh(', 'arccosh(', 'arctanh(')
+supported_symbols = ('(', ')', '+', '-', '/', '*', '^', '!')
+
 
 def basic_operations(operator, fir_int, sec_int=None):
     """
@@ -133,96 +140,15 @@ def number_parser(math_string):
     Returns
         List containing the elements from the string, with the numbers separated.
     """
-    pre_float_string = [i for i in math_string]  # decompose input string into elements in a list
-    pre_index_chain = nsu.indexer(pre_float_string)  # index the constructed list
+    # collect all values to split around and insert
+    collective_items = supported_natural_constants + supported_functions + supported_symbols
+    true_inserts = natural_constant_values + supported_functions + supported_symbols
 
-    # fix decimals and greater than 1-digit numbers
-    # temporary elements to be updated upon valid iterative
-    i = 0
-    temp_string = pre_float_string
-    temp_index_chain = pre_index_chain
-    while i < len(temp_string):  # iterate over the length of the 1-piece list
-        i0_val = temp_string[i]  # i'th value of the 1-piece list
-        ip1, im1, ip2, ip3, ip4 = i + 1, i - 1, i + 2, i + 3, i + 4  # define surrounding i´th values
-        im1_val = ip1_val = ip2_val = ip3_val = ip4_val = ''
-        try:  # try to find the surrounding values, if no such values, pass
-            im1_val = temp_string[im1]
-        except IndexError:
-            pass
-        try:
-            ip1_val = temp_string[ip1]
-            ip2_val = temp_string[ip2]
-            ip3_val = temp_string[ip3]
-            ip4_val = temp_string[ip4]
-        except IndexError:
-            pass
-
-        # if the current value is an int and next value is an int or a dot, concatenate the two 1-pieces and make a new
-        #   updated list with the concatenated element
-        if isinstance(nsu.string_to_float(i0_val), float) and (isinstance(nsu.string_to_int(ip1_val), int) or
-                                                               ip1_val == '.'):
-            temp_string = [''.join([i0_val, ip1_val]) if k == i else j for k, j in temp_index_chain if k != ip1]
-            temp_index_chain = nsu.indexer(temp_string)
-            continue  # break and restart loop with the updated list
-
-        # if the current string value's last element is a dot, and the next element is an int, concatenate and update
-        elif [h for h in i0_val][-1] == '.' and isinstance(nsu.string_to_int(ip1_val), int):
-            temp_string = [''.join([i0_val, ip1_val]) if k == i else j for k, j in temp_index_chain if k != ip1]
-            temp_index_chain = nsu.indexer(temp_string)
-            continue  # break and restart loop with the updated list
-
-        # if the current string value has a consecutive e-/+[int], join
-        elif i0_val == 'e' and (ip1_val in ('+', '-') or isinstance(nsu.string_to_int(ip1_val), int)):
-            if ip1_val in ('+', '-'):
-                temp_string = [''.join([im1_val, i0_val, ip1_val, ip2_val]) if k == i else j for k, j in
-                               temp_index_chain if k not in (im1, ip1, ip2)]
-            elif isinstance(nsu.string_to_int(ip1_val), int):
-                temp_string = [''.join([im1_val, i0_val, ip1_val]) if k == i else j for k, j in temp_index_chain if
-                               k not in (im1, ip1)]
-            temp_index_chain = nsu.indexer(temp_string)
-            i -= 1
-            continue  # break and restart loop with the updated list
-
-        # if the string contains key values pi, replace those with the value of pi
-        elif i0_val == 'p' and ip1_val == 'i':
-            temp_string = [str(np.pi) if k == i else j for k, j in temp_index_chain if k != ip1]
-            temp_index_chain = nsu.indexer(temp_string)
-            continue  # break and restart loop with the updated list
-
-        # replace natural constants
-        elif i0_val == '_':
-            if (ip1_val, ip2_val, ip3_val, ip4_val) == ('h', 'b', 'a', 'r'):
-                temp_string = [str(spc.hbar) if k == i else j for k, j in temp_index_chain if k not in (ip1, ip2, ip3,
-                                                                                                        ip4)]
-            elif (ip1_val, ip2_val) == ('N', 'A'):
-                temp_string = [str(spc.N_A) if k == i else j for k, j in temp_index_chain if k not in (ip1, ip2)]
-            elif (ip1_val, ip2_val) == ('m', 'e'):
-                temp_string = [str(spc.electron_mass) if k == i else j for k, j in temp_index_chain if k not in
-                               (ip1, ip2)]
-            elif (ip1_val, ip2_val) == ('m', 'p'):
-                temp_string = [str(spc.proton_mass) if k == i else j for k, j in temp_index_chain if k not in
-                               (ip1, ip2)]
-            elif ip1_val == 'c':
-                temp_string = [str(spc.c) if k == i else j for k, j in temp_index_chain if k != ip1]
-            elif ip1_val == 'h':
-                temp_string = [str(spc.h) if k == i else j for k, j in temp_index_chain if k != ip1]
-            elif ip1_val == 'e':
-                temp_string = [str(spc.e) if k == i else j for k, j in temp_index_chain if k != ip1]
-            elif ip1_val == 'R':
-                temp_string = [str(spc.R) if k == i else j for k, j in temp_index_chain if k != ip1]
-            elif ip1_val == 'k':
-                temp_string = [str(spc.k) if k == i else j for k, j in temp_index_chain if k != ip1]
-            else:
-                raise ValueError('Constant is not defined in parser.')
-            temp_index_chain = nsu.indexer(temp_string)
-
-        # if two negative signs are consecutive, change to a positive sign
-        elif i0_val == '-' and ip1_val == '-':
-            temp_string = ['+' if k == i else j for k, j in temp_index_chain if k != ip1]
-            temp_index_chain = nsu.indexer(temp_string)
-            continue
-        i += 1  # update iterator
-    return temp_string
+    # sort items and split-replace
+    sorted_items = nsu.list_sorter(collective_items + ('e+', 'e'), true_inserts + (['*', '10', '^'], ['*', '10', '^']),
+                                   reverse=True, otype='tuple')
+    split_input_string = nsu.multi_split(math_string, sorted_items[0], reps=sorted_items[1])
+    return split_input_string
 
 
 def ordered_parser(math_string, steps=False):
@@ -239,13 +165,13 @@ def ordered_parser(math_string, steps=False):
     Returns
         A float representing the result of the executed operations.
     """
-    parsed_numbers = number_parser(math_string)  # fix numbers in passed string
+
     # if the first value of the fixed list is a '-', append to the next value, preventing interpretation as an operator
-    if parsed_numbers[0] in ('-', '+'):
-        post_float_string = [''.join([parsed_numbers[0], parsed_numbers[1]]) if i == 0 else j for i, j in
-                             nsu.indexer(parsed_numbers) if i != 1]
+    if math_string in ('-', '+'):
+        post_float_string = [''.join([math_string[0], math_string[1]]) if i == 0 else j for i, j in
+                             nsu.indexer(math_string) if i != 1]
     else:
-        post_float_string = parsed_numbers
+        post_float_string = math_string
     post_index_chain = nsu.indexer(post_float_string)  # define index for the fixed string
 
     # fix negative numbers by creating a negative operator. Note that this prevents powers from interpreting all values
@@ -358,26 +284,23 @@ def product_parser(string, items, exclusions=None):
     return result_string
 
 
-def parser(math_string, steps=False, cprint='num', **kwargs):
+def parser(math_string, steps=False, cprint=True, **kwargs):
     """
     Takes care of the additional rules and conventions of mathematical operations. Handles parentheses along with
-    operators that require parentheses, such as trigonometric functions (sin, cos, tan, ...) and log, exp, etc.
+    operators that require parentheses, such as trigonometric functions (sin, cos, tan, ...) and log, exp, etc. This is
+    strictly meant to be a back-end function, and thus, for a user-friendly experience use modules.NumAn instead to
+    calculate.
 
     Parameters
         math_string : str
             The mathematical string to parse through the interpreter.
         steps : bool, optional
             If True, displays balances from the script, whilst performing the operations. The default is False.
-        cprint : str (or False), optional
-                Determines whether the computational result should be printed in the python console. There are four
-                options: 'num' will display input string with constants replaced with values, 'sym' will display input
-                string with constants as symbols, False will disable result print. The default is 'num'.
+        cprint : bool
+            Prints the result in the console along with the input expression reformatted to reflect how it was read by
+            the parser. The default is True.
 
     Keyword Arguments
-        true_string : str
-            A true input string, used when cprint is set, to swap symbols in etc. or simply for a prettier cprint.
-        unit_res : str
-            Allows for assigning a unit to the print result.
         sf : int
             Sets the significant figures of the result. Uses mpmath to do so. If set to None (which is default) no
             attempt will be made to set significant figures. Note that this only affects the result printed in the
@@ -386,176 +309,107 @@ def parser(math_string, steps=False, cprint='num', **kwargs):
     Returns
         The result from the performed operations on the given mathematical string as a float.
     """
-    # define temporary lists/values to be updated from while loop
-    # first define items for product_parser
-    constant_items = ('pi', '_hbar', '_NA', '_c', '_h', '_R', '_k', '_e', '_me', '_mp')
-    function_items = ('sinh(', 'cosh(', 'tanh(', 'exp(', 'sin(', 'cos(', 'tan(', 'ln(', 'rad(',
-                      'deg(', 'log(', 'sqrt(', 'arcsin(', 'arccos(', 'arctan(', 'arcsinh(', 'arccosh(', 'arctanh(')
-    collective_items = constant_items + function_items
 
-    temp_decom_string = product_parser(math_string, collective_items)
+    #  define items for product_parser
+    collective_items = supported_natural_constants + supported_functions  # collect items to place product around
+    product_string = product_parser(math_string, collective_items)  # 'explicitify' products
+    decom_string = number_parser(product_string)  # decompose string into bits that can be handled by basic parser
+
+    # make sure that brackets are done correctly in the right order
+    temp_decom_string = decom_string.copy()
     temp_index = nsu.indexer(temp_decom_string)
-    temp_bracket_idx = [[j] + [e] for j, e in temp_index if e in ('(', ')')]  # find and index open/close brackets
-    if steps:
-        print(math_string)
-    i = 0  # set starting iteration
+    temp_bracket_idx = [[j] + [e] for j, e in temp_index if e[-1] in ('(', ')')]  # find and index open/close brackets
+
+    i0 = 0  # set initial iteration
     while temp_bracket_idx:
         # if two consecutive brackets are a pair, execute operations through ordered_parser(), append the result to the
         #   given string, update it and reiterate. This ensures that the parentheses are read in the correct order
 
+        open_bracket = temp_bracket_idx[i0][1]
         try:  # check for missing closing parenthesis
-            closing_bracket = temp_bracket_idx[i + 1][1]
+            close_bracket = temp_bracket_idx[i0 + 1][1]
         except IndexError:
             raise ValueError('Missing closing bracket somewhere.')
 
-        if temp_bracket_idx[i][1] == '(' and closing_bracket == ')':
-            i0, i1 = temp_bracket_idx[i][0], temp_bracket_idx[i + 1][0]  # define current i'th values
+        if open_bracket[-1] == '(' and close_bracket == ')':
+            cur_open_id, cur_close_id = temp_bracket_idx[i0][0], temp_bracket_idx[i0 + 1][0]
+            bracket_excl = list(range(cur_open_id + 1, cur_close_id + 1))  # define the bracket clause as an exclusion
+            bracket_decom = temp_decom_string[cur_open_id + 1: cur_close_id]
+            pre_temp_result = ordered_parser(bracket_decom, steps)  # execute operations on the clause
 
-            # define consecutive -i's
-            im1, im2, im3, im4, im5, im6, im7 = i0 - 1, i0 - 2, i0 - 3, i0 - 4, i0 - 5, i0 - 6, i0 - 7
-            ip1 = i1 + 1  # define consecutive +i's
-            bracket_excl = list(range(i0 + 1, ip1))  # define the bracket clause as an exclusion
-            new_string = nsu.list_to_string(temp_decom_string[i0 + 1: i1])  # string consisting only of the clause
-            pre_temp_result = ordered_parser(new_string, steps)  # execute operations on the clause
-
-            # define temporary lists/values
-            id_excl = []
-            temp_result = pre_temp_result
-            im7_val = im6_val = im5_val = im4_val = im3_val = im2_val = im1_val = ip1_val = None
-            try:  # try to define values for the surrounding iterations, otherwise pass at position
-                im1_val = temp_decom_string[im1]
-                im2_val = temp_decom_string[im2]
-                im3_val = temp_decom_string[im3]
-                im4_val = temp_decom_string[im4]
-                im5_val = temp_decom_string[im5]
-                im6_val = temp_decom_string[im6]
-                im7_val = temp_decom_string[im7]
-            except IndexError:
-                pass
-            try:
-                ip1_val = temp_decom_string[ip1]
-            except IndexError:
-                pass
-
-            # if preceding iterations or upcoming operations leads to an identifier, perform special operation on the
-            #   clause, respecting order. From here, define temporary result to append, along with index for exclusions
-            if (im7_val, im6_val, im5_val, im4_val, im3_val, im2_val, im1_val) == ('a', 'r', 'c', 's', 'i', 'n', 'h'):
+            if open_bracket == 'arcsinh(':
                 temp_result = np.arcsinh(pre_temp_result)
-                id_excl = list(range(im7, i0))
-            elif (im7_val, im6_val, im5_val, im4_val, im3_val, im2_val, im1_val) == ('a', 'r', 'c', 'c', 'o', 's', 'h'):
+            elif open_bracket == 'arccosh(':
                 temp_result = np.arccosh(pre_temp_result)
-                id_excl = list(range(im7, i0))
-            elif (im7_val, im6_val, im5_val, im4_val, im3_val, im2_val, im1_val) == ('a', 'r', 'c', 't', 'a', 'n', 'h'):
+            elif open_bracket == 'arctanh(':
                 temp_result = np.arctanh(pre_temp_result)
-                id_excl = list(range(im7, i0))
-            elif (im6_val, im5_val, im4_val, im3_val, im2_val, im1_val) == ('a', 'r', 'c', 's', 'i', 'n'):
+            elif open_bracket == 'arcsin(':
                 temp_result = np.arcsin(pre_temp_result)
-                id_excl = list(range(im6, i0))
-            elif (im6_val, im5_val, im4_val, im3_val, im2_val, im1_val) == ('a', 'r', 'c', 'c', 'o', 's'):
+            elif open_bracket == 'arccos(':
                 temp_result = np.arccos(pre_temp_result)
-                id_excl = list(range(im6, i0))
-            elif (im6_val, im5_val, im4_val, im3_val, im2_val, im1_val) == ('a', 'r', 'c', 't', 'a', 'n'):
-                temp_result = np.tan(pre_temp_result)
-                id_excl = list(range(im6, i0))
-            elif (im4_val, im3_val, im2_val, im1_val) == ('s', 'i', 'n', 'h'):
+            elif open_bracket == 'arctan(':
+                temp_result = np.arctan(pre_temp_result)
+            elif open_bracket == 'sinh(':
                 temp_result = np.sinh(pre_temp_result)
-                id_excl = list(range(im4, i0))
-            elif (im4_val, im3_val, im2_val, im1_val) == ('c', 'o', 's', 'h'):
+            elif open_bracket == 'cosh(':
                 temp_result = np.cosh(pre_temp_result)
-                id_excl = list(range(im4, i0))
-            elif (im4_val, im3_val, im2_val, im1_val) == ('t', 'a', 'n', 'h'):
+            elif open_bracket == 'tanh(':
                 temp_result = np.tanh(pre_temp_result)
-                id_excl = list(range(im4, i0))
-            elif (im4_val, im3_val, im2_val, im1_val) == ('s', 'q', 'r', 't'):
+            elif open_bracket == 'sqrt(':
                 temp_result = pre_temp_result ** (1 / 2)
-                id_excl = list(range(im4, i0))
-            elif (im3_val, im2_val, im1_val) == ('e', 'x', 'p'):
+            elif open_bracket == 'exp(':
                 temp_result = np.exp(pre_temp_result)
-                id_excl = list(range(im3, i0))
-            elif (im3_val, im2_val, im1_val) == ('l', 'o', 'g'):
-                if pre_temp_result == 0:
-                    raise ValueError('Parser does not support infinity values, log10(0) = -inf.')
-                else:
-                    temp_result = np.log10(pre_temp_result)
-                id_excl = list(range(im3, i0))
-            elif (im3_val, im2_val, im1_val) == ('s', 'i', 'n'):
-                # fix sin(n*pi) numerical variation for n in NN
-                if pre_temp_result == 0:
-                    temp_result = 0
-                else:
-                    arbitrary_x = sp.symbols('arbitraryX')
-                    equation_sin_pi_solution = sp.solve(pre_temp_result / arbitrary_x - np.pi)
-                    if isinstance(nsu.float_to_int(equation_sin_pi_solution[0]), int):
-                        temp_result = 0
-                    else:
-                        temp_result = np.sin(pre_temp_result)
-                id_excl = list(range(im3, i0))
-            elif (im3_val, im2_val, im1_val) == ('c', 'o', 's'):
+            elif open_bracket == 'log(':
+                temp_result = np.log(pre_temp_result)
+            elif open_bracket == 'sin(':
+                temp_result = np.sin(pre_temp_result)
+            elif open_bracket == 'cos(':
                 temp_result = np.cos(pre_temp_result)
-                id_excl = list(range(im3, i0))
-            elif (im3_val, im2_val, im1_val) == ('t', 'a', 'n'):
+            elif open_bracket == 'tan(':
                 temp_result = np.tan(pre_temp_result)
-                id_excl = list(range(im3, i0))
-            elif (im3_val, im2_val, im1_val) == ('d', 'e', 'g'):
-                temp_result = 360 / (2 * sp.pi) * pre_temp_result
-                id_excl = list(range(im3, i0))
-            elif (im3_val, im2_val, im1_val) == ('r', 'a', 'd'):
-                temp_result = (2 * sp.pi) / 360 * pre_temp_result
-                id_excl = list(range(im3, i0))
-            elif (im2_val, im1_val) == ('l', 'n'):
+            elif open_bracket == 'deg(':
+                temp_result = 360 / (2 * np.pi) * pre_temp_result
+            elif open_bracket == 'rad(':
+                temp_result = (2 * np.pi) / 360 * pre_temp_result
+            elif open_bracket == 'ln(':
                 if pre_temp_result == 0:
                     raise ValueError('Parser does not support infinity values, ln(0) = -inf.')
                 else:
                     temp_result = np.log(pre_temp_result)
-                id_excl = list(range(im2, i0))
-
-            temp_excel = bracket_excl + id_excl  # define all needed exclusions in a list
+            else:
+                temp_result = pre_temp_result
 
             # update the temporary string, index, and bracket index and reiterate
-            temp_decom_string = [temp_result if k == i0 else j for k, j in temp_index if k not in temp_excel]
-            if steps:
-                print(nsu.list_to_string(temp_decom_string))
+            temp_decom_string = [temp_result if k == cur_open_id else j for k, j in temp_index if k not in bracket_excl]
             temp_index = nsu.indexer(temp_decom_string)
-            temp_bracket_idx = [[j] + [e] for j, e in temp_index if e in ('(', ')')]
-            i -= 1  # reset iteration
+            temp_bracket_idx = [[j] + [e] for j, e in temp_index if str(e)[-1] in ('(', ')')]
+            i0 -= 1  # reset iteration
             continue
-        i += 1
+        i0 += 1
     else:  # if no brackets are present in iterated string, perform operations as usual per ordered_parser()
-        new_string = nsu.list_to_string(temp_decom_string)
-        parsed_string = ordered_parser(new_string, steps)
+        parsed_string = ordered_parser(temp_decom_string, steps)
         int_fixed_string = nsu.float_to_int(parsed_string)
 
     # auto-print if prompted
     if cprint:
 
-        # check whether a true input string has been given, along with added unit result and significant figures
-        true_string, unit_res, sf = math_string, '', None
-        if 'true_string' in kwargs.keys():
-            true_string = kwargs.get('true_string')
-        if 'unit_res' in kwargs.keys():
-            unit_res = kwargs.get('unit_res')
+        # check if significant figures was set
+        sf = None
         if 'sf' in kwargs.keys():
             sf = kwargs.get('sf')
 
-        # define specific set of replacement keys/values depending on cprint type
-        if cprint == 'num':
-            replacement_keys = ('pi', '_hbar', '_NA', '_c', '_h', '*', '_R', '_k', '_e', '_me', '_mp')
-            replacement_vals = ('π', 'ħ', 'Nᴀ', 'c', 'h', '·', 'R', 'k', 'e', 'mₑ', 'mₚ')
-        elif cprint == 'sym':
-            replacement_keys = ['_hbar', '_NA', '_c', '_h', '*', '_R', '_k', '_e', '_me', '_mp'] + \
-                               nsu.alphabetSequenceGreekLetters + nsu.alphabetSequenceGreekLettersCap
-            replacement_vals = ['ħ', 'Nᴀ', 'c', 'h', '·', 'R', 'kᴮ', 'e', 'mₑ', 'mₚ'] + \
-                               nsu.alphabetSequenceGreek + nsu.alphabetSequenceGreekCap
-        else:
-            raise ValueError(f'Computation print type \'{cprint}\' is not supported.')
+        # define specific set of replacement keys/values to prettify constants in expression
+        replacement_keys = ('pi', '_hbar', '_NA', '_c', '_h', '*', '_R', '_k', '_e', '_me', '_mp')
+        replacement_vals = ('π', 'ħ', 'Nᴀ', 'cᵥ', 'hₚ', '·', 'Rᶢ', 'kᴮ', 'eᶜ', 'mₑ', 'mₚ')
 
         # sort the replacements with their keys, replace them and print
         sorted_replacements = nsu.list_sorter(replacement_keys, replacement_vals, reverse=True, otype='tuple')
-        pretty_string = nsu.replace(sorted_replacements[0], sorted_replacements[1], true_string, 'amu')
+        pretty_string = nsu.replace(sorted_replacements[0], sorted_replacements[1], product_string)
+
         if sf:
             with mpmath.workdps(sf):
                 res = mpmath.mpf(int_fixed_string)
-                print(f'Result: {pretty_string} = {str(nsu.float_to_int(res)) + unit_res}')
+                print(f'Result: {pretty_string} = {nsu.float_to_int(res)}')
         else:
-            print(f'Result: {pretty_string} = {str(int_fixed_string) + unit_res}')
+            print(f'Result: {pretty_string} = {int_fixed_string}')
     return int_fixed_string
